@@ -113,30 +113,22 @@ static GObjectClass *parent_class = NULL;
 static void
 brasero_session_settings_clean (BraseroSessionSetting *settings)
 {
-	if (settings->image) {
+	if (settings->image)
 		g_free (settings->image);
-		settings->image = NULL;
-	}
 
-	if (settings->toc) {
+	if (settings->toc)
 		g_free (settings->toc);
-		settings->toc = NULL;
-	}
 
-	if (settings->tmpdir) {
+	if (settings->tmpdir)
 		g_free (settings->tmpdir);
-		settings->tmpdir = NULL;
-	}
 
-	if (settings->label) {
+	if (settings->label)
 		g_free (settings->label);
-		settings->label = NULL;
-	}
 
-	if (settings->burner) {
+	if (settings->burner)
 		nautilus_burn_drive_unref (settings->burner);
-		settings->burner = NULL;
-	}
+
+	memset (settings, 0, sizeof (BraseroSessionSetting));
 }
 
 void
@@ -271,9 +263,9 @@ brasero_burn_session_add_track (BraseroBurnSession *self,
 		brasero_burn_session_start_src_drive_monitoring (self);
 
 		/* if (!brasero_track_type_equal (priv->input, &new_type)) */
-			g_signal_emit (self,
-				       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
-				       0);
+		g_signal_emit (self,
+			       brasero_burn_session_signals [INPUT_CHANGED_SIGNAL],
+			       0);
 
 		return BRASERO_BURN_OK;
 	}
@@ -466,7 +458,7 @@ brasero_burn_session_get_rate (BraseroBurnSession *self)
 		return 0;
 
 	max_rate = NCB_MEDIA_GET_MAX_WRITE_RATE (priv->settings->burner);
-	if (priv->settings->rate > 0)
+	if (priv->settings->rate <= 0)
 		return max_rate;
 	else
 		return MIN (max_rate, priv->settings->rate);
@@ -1392,9 +1384,7 @@ brasero_burn_session_get_log_path (BraseroBurnSession *self)
 gboolean
 brasero_burn_session_start (BraseroBurnSession *self)
 {
-	gchar *start_message;
 	BraseroTrackType type;
-	BraseroImageFormat output;
 	BraseroBurnSessionPrivate *priv;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), FALSE);
@@ -1420,28 +1410,25 @@ brasero_burn_session_start (BraseroBurnSession *self)
 		return FALSE;
 	}
 
-	output = brasero_burn_session_get_output_format (self);
-	start_message = g_strdup_printf ("Session starting:\n"
-					 "\tflags\t\t\t= %i \n"
-					 "\tmedia type\t= %s\n"
-					 "\tspeed\t\t= %lli\n"
-					 "\ttrack format\t= %i\n"
-					 "\toutput\t\t= %s"
-					 "\tnumber of copies = %i\n",
-					 priv->settings->flags,
-					 NCB_MEDIA_GET_TYPE_STRING (priv->settings->burner),
-					 priv->settings->rate,
-					 output,
-					 priv->settings->image ? priv->settings->image:"none",
-					 priv->settings->num_copies);
 
-	BRASERO_BURN_LOG (start_message);
+	BRASERO_BURN_LOG ("Session starting:");
 
 	brasero_burn_session_get_input_type (self, &type);
-	BRASERO_BURN_LOG_TYPE (&type, "Input:\t");
+	BRASERO_BURN_LOG_TYPE (&type, "Input\t=");
+
+	BRASERO_BURN_LOG_FLAGS (priv->settings->flags, "flags\t=");
+
+	if (!brasero_burn_session_is_dest_file (self)) {
+		BRASERO_BURN_LOG_DISC_TYPE (NCB_MEDIA_GET_STATUS (priv->settings->burner), "media type\t=");
+		BRASERO_BURN_LOG ("speed\t= %i", priv->settings->rate);
+		BRASERO_BURN_LOG ("number of copies\t= %i", priv->settings->num_copies);
+	}
+	else {
+		type.type = BRASERO_TRACK_TYPE_IMAGE;
+		type.subtype.img_format = brasero_burn_session_get_output_format (self);
+		BRASERO_BURN_LOG_TYPE (&type, "output format\t=");
+	}
 
-	brasero_burn_session_logv (self, start_message, NULL);
-	g_free (start_message);
 	return TRUE;
 }
 
@@ -1459,6 +1446,86 @@ brasero_burn_session_stop (BraseroBurnSession *self)
 	}
 }
 
+/**
+ * Misc
+ */
+
+gchar *
+brasero_burn_session_get_config_key (BraseroBurnSession *self,
+				     const gchar *property)
+{
+	NautilusBurnDrive *drive;
+	gchar *display_name;
+	gchar *key = NULL;
+	gchar *disc_type;
+
+	drive = brasero_burn_session_get_burner (self);
+	if (!drive)
+		return NULL;
+
+	if (NCB_MEDIA_GET_STATUS (drive) == BRASERO_MEDIUM_NONE)
+		return NULL;
+	
+	/* make sure display_name doesn't contain any forbidden characters */
+	display_name = nautilus_burn_drive_get_name_for_display (drive);
+	g_strdelimit (display_name, " +()", '_');
+
+	disc_type = g_strdup (NCB_MEDIA_GET_TYPE_STRING (drive));
+	if (!disc_type) {
+		g_free (display_name);
+		return NULL;
+	}
+
+	g_strdelimit (disc_type, " +()", '_');
+
+	switch (brasero_burn_session_get_input_type (self, NULL)) {
+	case BRASERO_TRACK_TYPE_NONE:
+		key = g_strdup_printf ("%s/%s/none_%s/%s",
+				       BRASERO_DRIVE_PROPERTIES_KEY,
+				       display_name,
+				       disc_type,
+				       property);
+		break;
+	case BRASERO_TRACK_TYPE_DISC:
+		key = g_strdup_printf ("%s/%s/disc_%s/%s",
+				       BRASERO_DRIVE_PROPERTIES_KEY,
+				       display_name,
+				       disc_type,
+				       property);
+		break;
+
+	case BRASERO_TRACK_TYPE_DATA:
+		key = g_strdup_printf ("%s/%s/data_%s/%s",
+				       BRASERO_DRIVE_PROPERTIES_KEY,
+				       display_name,
+				       disc_type,
+				       property);
+		break;
+
+	case BRASERO_TRACK_TYPE_IMAGE:
+		key = g_strdup_printf ("%s/%s/image_%s/%s",
+				       BRASERO_DRIVE_PROPERTIES_KEY,
+				       display_name,
+				       disc_type,
+				       property);
+		break;
+
+	case BRASERO_TRACK_TYPE_AUDIO:
+		key = g_strdup_printf ("%s/%s/audio_%s/%s",
+				       BRASERO_DRIVE_PROPERTIES_KEY,
+				       display_name,
+				       disc_type,
+				       property);
+		break;
+	default:
+		break;
+	}
+
+	g_free (display_name);
+	g_free (disc_type);
+	return key;
+}
+
 /**
  *
  */
@@ -1545,6 +1612,8 @@ brasero_burn_session_finalize (GObject *object)
 		priv->wrong_checksums = NULL;
 	}
 
+	brasero_session_settings_clean (priv->settings);
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
