@@ -50,13 +50,14 @@
 #endif
 
 #include "brasero-app.h"
-#include "menu.h"
+#include "brasero-menu.h"
 #include "brasero-multi-dnd.h"
-#include "blank-dialog.h"
+#include "brasero-blank-dialog.h"
 #include "brasero-sum-dialog.h"
 #include "brasero-session.h"
 #include "brasero-project-manager.h"
 #include "brasero-ncb.h"
+#include "brasero-pref.h"
 #include "burn-debug.h"
 
 static GConfClient *client;
@@ -67,6 +68,7 @@ gchar **audio_project;
 gchar **data_project;
 gint copy_project;
 gint empty_project;
+gint disc_blank;
 gint is_escaped;
 gint open_ncb;
 gint debug;
@@ -96,6 +98,10 @@ static const GOptionEntry options [] = {
          N_("Force brasero to display the project selection page"),
           NULL },
 
+	{ "blank", 'b', 0, G_OPTION_ARG_NONE, &disc_blank,
+	  N_("Open the blank disc dialog"),
+	  NULL },
+
 	{ "ncb", 'n', 0, G_OPTION_ARG_NONE, &open_ncb,
 	  N_("Open a data project with the contents of nautilus-cd-burner"),
           NULL },
@@ -135,8 +141,7 @@ static const GOptionEntry options [] = {
 static gboolean
 on_delete_cb (GtkWidget *window, GdkEvent *event, BraseroApp *app)
 {
-	brasero_session_save (app, TRUE);
-	return FALSE;
+	return brasero_session_save (app, TRUE, TRUE);
 }
 
 static gboolean
@@ -149,7 +154,9 @@ on_destroy_cb (GtkWidget *window, GdkEvent *event, BraseroApp *app)
 void
 on_exit_cb (GtkAction *action, BraseroApp *app)
 {
-	brasero_session_save (app, TRUE);
+	if (brasero_session_save (app, TRUE, TRUE))
+		return;
+
 	gtk_widget_destroy (app->mainwin);
 }
 
@@ -185,6 +192,24 @@ on_integrity_check_cb (GtkAction *action, BraseroApp *app)
 	gtk_widget_show_all (dialog);
 }
 
+void
+on_prefs_cb (GtkAction *action, BraseroApp *app)
+{
+	GtkWidget *dialog;
+	GtkWidget *toplevel;
+
+	dialog = brasero_pref_new ();
+	toplevel = gtk_widget_get_toplevel (app->mainwin);
+
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+
+	gtk_widget_show_all (dialog);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+}
+
 void
 on_disc_info_cb (GtkAction *button, BraseroApp *app)
 {
@@ -218,7 +243,7 @@ on_about_cb (GtkAction *action, BraseroApp *app)
                 N_("You should have received a copy of the GNU General "
 		   "Public License along with Brasero; if not, write "
 		   "to the Free Software Foundation, Inc., "
-                   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
+                   "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA"),
 		NULL
         };
 
@@ -236,7 +261,7 @@ on_about_cb (GtkAction *action, BraseroApp *app)
 			       "name", "Brasero",
 			       "comments", comments,
 			       "version", VERSION,
-			       "copyright", "Copyright © 2006 Philippe Rouquier",
+			       "copyright", "Copyright © 2005-2007 Philippe Rouquier",
 			       "authors", authors,
 			       "documenters", documenters,
 			       "website", "http://www.gnome.org/projects/brasero",
@@ -289,11 +314,35 @@ static void
 brasero_app_recent_open (GtkRecentChooser *chooser,
 			 BraseroApp *app)
 {
+	gchar *uri;
     	const gchar *mime;
     	GtkRecentInfo *item;
+	GtkRecentManager *manager;
 
-    	item = gtk_recent_chooser_get_current_item (chooser);
-    	mime = gtk_recent_info_get_mime_type (item);
+	/* This is a workaround since following code doesn't work */
+	/*
+    	item = gtk_recent_chooser_get_current_item (GTK_RECENT_CHOOSER (chooser));
+	if (!item)
+		return;
+	*/
+
+	uri = gtk_recent_chooser_get_current_uri (GTK_RECENT_CHOOSER (chooser));
+	if (!uri)
+		return;
+
+	manager = gtk_recent_manager_get_default ();
+	item = gtk_recent_manager_lookup_item (manager, uri, NULL);
+	g_free (uri);
+
+	if (!item)
+		return;
+
+	mime = gtk_recent_info_get_mime_type (item);
+
+	if (!mime) {
+		g_warning ("Unrecognized mime type");
+		return;
+	}
 
 	if (!strcmp (mime, "application/x-brasero")) {
 		BRASERO_PROJECT_OPEN_URI (app,
@@ -308,38 +357,116 @@ brasero_app_recent_open (GtkRecentChooser *chooser,
 					  brasero_project_manager_iso,
 					  gtk_recent_info_get_uri (item));
 	}
+	else
+		g_warning ("Unknown type of file can't open");
+
+	gtk_recent_info_unref (item);
 }
 
 static void
-brasero_app_add_recent (BraseroApp *app)
+brasero_app_add_recent (BraseroApp *app,
+			GtkActionGroup *group)
 {
 	GtkRecentManager *recent;
 	GtkRecentFilter *filter;
-	GtkWidget *submenu;
-	GtkWidget *menu;
+	GtkAction *action;
 
-	/* add recent files */
 	recent = gtk_recent_manager_get_default ();
- 	gtk_recent_manager_set_limit (recent, 20);
-
-	submenu = gtk_recent_chooser_menu_new_for_manager (recent);
-	g_signal_connect (submenu,
-                  	  "item_activated",
-			  G_CALLBACK (brasero_app_recent_open),
-			  app);
-
+	action = gtk_recent_action_new_for_manager ("RecentProjects",
+						    _("_Recent Projects"),
+						    _("Display the projects recently opened"),
+						    NULL,
+						    recent);
 	filter = gtk_recent_filter_new ();
+
 	gtk_recent_filter_set_name (filter, _("Brasero projects"));
 	gtk_recent_filter_add_mime_type (filter, "application/x-brasero");
 	gtk_recent_filter_add_mime_type (filter, "application/x-cd-image");
 	gtk_recent_filter_add_mime_type (filter, "application/x-cdrdao-toc");
 	gtk_recent_filter_add_mime_type (filter, "application/x-toc");
 	gtk_recent_filter_add_mime_type (filter, "application/x-cue");
-	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (submenu), filter);
-	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (submenu), TRUE);
+	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (action), filter);
+	gtk_recent_chooser_set_filter (GTK_RECENT_CHOOSER (action), filter);
 
-	menu = gtk_ui_manager_get_widget (app->manager, "/menubar/ProjectMenu/Recent");
- 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);    
+	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (action), TRUE);
+
+	gtk_recent_chooser_set_limit (GTK_RECENT_CHOOSER (action), 20);
+
+	gtk_recent_chooser_set_show_tips (GTK_RECENT_CHOOSER (action), TRUE);
+
+	gtk_recent_chooser_set_show_icons (GTK_RECENT_CHOOSER (action), TRUE);
+
+	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (action), GTK_RECENT_SORT_MRU);
+
+	gtk_action_group_add_action (group, action);
+	g_object_unref (action);
+	g_signal_connect (action,
+			  "item-activated",
+			  G_CALLBACK (brasero_app_recent_open),
+			  app);
+}
+
+static void
+brasero_menu_item_selected_cb (GtkMenuItem *proxy,
+			       BraseroApp *app)
+{
+	GtkAction *action;
+	gchar *message;
+
+	action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+	g_return_if_fail (action != NULL);
+
+	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+	if (message) {
+		gtk_statusbar_push (GTK_STATUSBAR (app->statusbar),
+				    app->tooltip_ctx,
+				    message);
+		g_free (message);
+	}
+}
+
+static void
+brasero_menu_item_deselected_cb (GtkMenuItem *proxy,
+				 BraseroApp *app)
+{
+	gtk_statusbar_pop (GTK_STATUSBAR (app->statusbar),
+			   app->tooltip_ctx);
+}
+
+static void
+brasero_connect_ui_manager_proxy_cb (GtkUIManager *manager,
+				     GtkAction *action,
+				     GtkWidget *proxy,
+				     BraseroApp *app)
+{
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	g_signal_connect (proxy,
+			  "select",
+			  G_CALLBACK (brasero_menu_item_selected_cb),
+			  app);
+	g_signal_connect (proxy,
+			  "deselect",
+			  G_CALLBACK (brasero_menu_item_deselected_cb),
+			  app);
+}
+
+static void
+brasero_disconnect_ui_manager_proxy_cb (GtkUIManager *manager,
+					GtkAction *action,
+					GtkWidget *proxy,
+					BraseroApp *app)
+{
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	g_signal_handlers_disconnect_by_func (proxy,
+					      G_CALLBACK (brasero_menu_item_selected_cb),
+					      app);
+	g_signal_handlers_disconnect_by_func (proxy,
+					      G_CALLBACK (brasero_menu_item_deselected_cb),
+					      app);
 }
 
 static BraseroApp *
@@ -365,6 +492,7 @@ brasero_app_create_app (void)
 
 	/* status bar to display the size of selected files */
 	app->statusbar = gtk_statusbar_new ();
+	app->tooltip_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (app->statusbar), "tooltip_info");
 	gnome_app_set_statusbar (GNOME_APP (app->mainwin), app->statusbar);
 
 	/* window contents */
@@ -377,8 +505,14 @@ brasero_app_create_app (void)
 
 	/* menu and toolbar */
 	app->manager = gtk_ui_manager_new ();
-	brasero_project_manager_register_menu (BRASERO_PROJECT_MANAGER (app->contents),
-					       app->manager);
+	g_signal_connect (app->manager,
+			  "connect-proxy",
+			  G_CALLBACK (brasero_connect_ui_manager_proxy_cb),
+			  app);
+	g_signal_connect (app->manager,
+			  "disconnect-proxy",
+			  G_CALLBACK (brasero_disconnect_ui_manager_proxy_cb),
+			  app);
 
 	action_group = gtk_action_group_new ("MenuActions");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
@@ -389,17 +523,20 @@ brasero_app_create_app (void)
 
 	gtk_ui_manager_insert_action_group (app->manager, action_group, 0);
 
+	brasero_app_add_recent (app, action_group);
+
 	if (!gtk_ui_manager_add_ui_from_string (app->manager, description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
 	}
 
+	brasero_project_manager_register_ui (BRASERO_PROJECT_MANAGER (app->contents),
+					     app->manager);
+
 	gtk_ui_manager_ensure_update (app->manager);
 	menubar = gtk_ui_manager_get_widget (app->manager, "/menubar");
 	gnome_app_set_menus (GNOME_APP (app->mainwin), GTK_MENU_BAR (menubar));
 
-	brasero_app_add_recent (app);
- 
 	/* add accelerators */
 	accel_group = gtk_ui_manager_get_accel_group (app->manager);
 	gtk_window_add_accel_group (GTK_WINDOW (app->mainwin), accel_group);
@@ -444,6 +581,8 @@ brasero_app_parse_options (BraseroApp *app)
 		nb ++;
 	if (data_project)
 		nb ++;
+	if (disc_blank)
+	  	nb ++;
 	if (open_ncb)
 		nb ++;
 
@@ -482,6 +621,9 @@ brasero_app_parse_options (BraseroApp *app)
 	else if (data_project) {
 		BRASERO_PROJECT_OPEN_LIST (app, brasero_project_manager_data, files);
 	}
+	else if (disc_blank) {
+		on_erase_cb (NULL, app);
+	}
 	else if (open_ncb) {
 		GSList *list = NULL;
 		gchar **iter;
@@ -568,12 +710,11 @@ main (int argc, char **argv)
 	notify_init (PACKAGE);
 #endif
 
-	nautilus_burn_init ();
 	gnome_vfs_init ();
 	gst_init (&argc, &argv);
 
 	brasero_burn_set_debug (debug);
-	NCB_INIT ();
+	brasero_burn_library_init ();
 
 	client = gconf_client_get_default ();
 
@@ -590,8 +731,10 @@ main (int argc, char **argv)
 
 	gtk_widget_show (app->mainwin);
 
+	/* debug */
 	gtk_main ();
 
+	brasero_burn_library_shutdown ();
 	brasero_session_disconnect (app);
 	g_object_unref (program);
 	g_free (app);
